'
'  pb_reg.bas
'
'  Registry functions for Power Basic 32-bit
'
'  Created by Don Dickinson
'  ddickinson@usinternet.com
'  dickinson.basicguru.com
'  Oct, 1999
'
'  This code is provided for use by anyone who
'  deems it useful. Use of said code is implied
'  agreement that DO NOT hold the author, Don
'  Dickinson, liable for any effects or side-effects
'  of use or mis-use of this code.
'
'  Requires WIN32API.INC - not included here.
'
'  pbRegOpenSection
'  pbRegClose
'  pbRegSetString
'  pbRegGetString
'  pbRegGetSubkeys
'
#IF NOT %DEF(%PB_REG_BAS)
%PB_REG_BAS = 1

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbRegOpenSection
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION pbRegOpenSection (BYVAL Key AS LONG, Section AS ASCIIZ) AS DWORD

	 DIM hKey AS LONG
	 DIM Result AS LONG

	 ' Create the section
	 IF RegCreateKeyEx(Key, BYVAL VARPTR(Section), 0, "", %REG_OPTION_NON_VOLATILE,%KEY_ALL_ACCESS , BYVAL %NULL, hKey, Result) <> %ERROR_SUCCESS THEN
			FUNCTION = 0
			EXIT FUNCTION
	 END IF

	 '- Return the registry key handle
	 FUNCTION = hKey

END FUNCTION

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbRegClose
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION pbRegClose (BYVAL hKey AS LONG) AS LONG

	 RegCloseKey hKey

END FUNCTION

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbRegSetString
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION pbRegSetString (BYVAL hKey AS LONG, Entry AS ASCIIZ, Value AS ASCIIZ) AS LONG

	 '- Save the value for the entry
	 IF LEN(Value) THEN
			FUNCTION = RegSetValueEx(hKey, BYVAL VARPTR(Entry), 0, %REG_SZ, Value, LEN(Value) + 1)
	 ELSE
			FUNCTION = RegSetValueEx(hKey, BYVAL VARPTR(Entry), 0, %REG_SZ, BYVAL %NULL, 0)
	 END IF

END FUNCTION

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbRegGetString
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION pbRegGetString (BYVAL hKey AS LONG, zKey AS ASCIIZ) AS STRING

	 DIM zBuffer AS ASCIIZ * 1000

	 IF RegQueryValueEx(hKey, BYVAL VARPTR(zKey), 0, %REG_SZ, zBuffer, SIZEOF(zBuffer) - 1) = %ERROR_SUCCESS THEN
			FUNCTION = TRIM$(zBuffer)
	 ELSE
			FUNCTION = ""
	 END IF

END FUNCTION

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbRegGetSubkeys
'
'  Fills subKeys() with an array of keys
'  Returns the number of keys in the array or -1 on error
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FUNCTION pbGetSubKeys (BYVAL hKey AS LONG, zKey AS ASCIIZ,subKeys() AS STRING) AS LONG

	 DIM openKey AS LONG
	 DIM iCount AS LONG
	 DIM iLen AS LONG
	 'DIM Result AS LONG
	 DIM ft AS FILETIME
	 DIM zBuf AS ASCIIZ * 2000

	 IF RegOpenKeyEx(hKey, BYVAL VARPTR(zKey), 0,%KEY_ALL_ACCESS, openKey) <> %ERROR_SUCCESS THEN
			FUNCTION = -1
			EXIT FUNCTION
	 END IF

	 DO
			iLen = 2000
			IF RegEnumKeyEx(openKey, iCount, BYVAL VARPTR(zBuf), iLen, 0, BYVAL 0, BYVAL 0, ft) <> %ERROR_SUCCESS THEN
				 EXIT DO
			END IF
			INCR iCount
REM       REDIM PRESERVE subKeys(0 TO iCount) AS STRING
			subKeys(iCount) = TRIM$(zBuf)
	 LOOP

	 pbRegClose hKey

	 FUNCTION = iCount

END FUNCTION
#ENDIF
